@@ -59,7 +59,7 @@ fn create_context() -> Result<SessionContext> {
 /// In this example we will declare a single-type, single return type UDF that exponentiates f64, a^b
 #[tokio::main]
 async fn main() -> Result<()> {
-    let mut ctx = create_context()?;
+    let ctx = create_context()?;
 
     // First, declare the actual implementation of the calculation
     let pow = |args: &[ArrayRef]| {
@@ -122,7 +122,7 @@ async fn main() -> Result<()> {
     let expr = pow.call(vec![col("a"), col("b")]);
 
     // get a DataFrame from the context
-    let df = ctx.table("t")?;
+    let df = ctx.table("t").await?;
 
     // if we do not have `pow` in the scope and we registered it, we can get it from the registry
     let pow = df.registry().udf("pow")?;
