@@ -21,7 +21,7 @@ use std::any::Any;
 use std::sync::Arc;
 
 use async_trait::async_trait;
-use datafusion_common::Statistics;
+use datafusion_common::{DataFusionError, Statistics};
 use datafusion_expr::{CreateExternalTable, LogicalPlan};
 pub use datafusion_expr::{TableProviderFilterPushDown, TableType};
 
@@ -97,6 +97,16 @@ pub trait TableProvider: Sync + Send {
     fn statistics(&self) -> Option<Statistics> {
         None
     }
+
+    /// Insert into this table
+    async fn insert_into(
+        &self,
+        _state: &SessionState,
+        _input: &LogicalPlan,
+    ) -> Result<()> {
+        let msg = "Insertion not implemented for this table".to_owned();
+        Err(DataFusionError::NotImplemented(msg))
+    }
 }
 
 /// A factory which creates [`TableProvider`]s at runtime given a URL.
