@@ -23,6 +23,7 @@ use crate::config::ConfigOptions;
 use crate::error::Result;
 use crate::physical_plan::sorts::sort::SortExec;
 use crate::physical_plan::{with_new_children_if_necessary, ExecutionPlan};
+use datafusion_physical_expr::utils::ordering_satisfy;
 use datafusion_physical_expr::PhysicalSortExpr;
 use std::sync::Arc;
 
@@ -48,22 +49,21 @@ pub fn optimize_children(
     }
 }
 
-/// Util function to add SortExec above child
-/// preserving the original partitioning
-pub fn add_sort_above_child(
-    child: &Arc<dyn ExecutionPlan>,
+/// This utility function adds a `SortExec` above an operator according to the
+/// given ordering requirements while preserving the original partitioning.
+pub fn add_sort_above(
+    node: &mut Arc<dyn ExecutionPlan>,
     sort_expr: Vec<PhysicalSortExpr>,
-) -> Result<Arc<dyn ExecutionPlan>> {
-    let new_child = if child.output_partitioning().partition_count() > 1 {
-        Arc::new(SortExec::new_with_partitioning(
-            sort_expr,
-            child.clone(),
-            true,
-            None,
-        )) as Arc<dyn ExecutionPlan>
-    } else {
-        Arc::new(SortExec::try_new(sort_expr, child.clone(), None)?)
-            as Arc<dyn ExecutionPlan>
-    };
-    Ok(new_child)
+) -> Result<()> {
+    // If the ordering requirement is already satisfied, do not add a sort.
+    if !ordering_satisfy(node.output_ordering(), Some(&sort_expr), || {
+        node.equivalence_properties()
+    }) {
+        *node = Arc::new(if node.output_partitioning().partition_count() > 1 {
+            SortExec::new_with_partitioning(sort_expr, node.clone(), true, None)
+        } else {
+            SortExec::try_new(sort_expr, node.clone(), None)?
+        }) as _
+    }
+    Ok(())
 }
