@@ -653,10 +653,10 @@ pub mod metrics;
 pub mod planner;
 pub mod projection;
 pub mod repartition;
-pub mod rewrite;
 pub mod sorts;
 pub mod stream;
 pub mod streaming;
+pub mod tree_node;
 pub mod udaf;
 pub mod union;
 pub mod unnest;
