@@ -139,7 +139,7 @@ impl ContextWithParquet {
         let file = match unit {
             Unit::RowGroup => make_test_file_rg(scenario).await,
             Unit::Page => {
-                let config = config.config_options_mut();
+                let config = config.options_mut();
                 config.execution.parquet.enable_page_index = true;
                 make_test_file_page(scenario).await
             }
