@@ -1313,6 +1313,24 @@ async fn test_extract_date_part() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn test_extract_epoch() -> Result<()> {
+    test_expression!(
+        "extract(epoch from '1870-01-01T07:29:10.256'::timestamp)",
+        "-3155646649.744"
+    );
+    test_expression!(
+        "extract(epoch from '2000-01-01T00:00:00.000'::timestamp)",
+        "946684800.0"
+    );
+    test_expression!(
+        "extract(epoch from to_timestamp('2000-01-01T00:00:00+00:00'))",
+        "946684800.0"
+    );
+    test_expression!("extract(epoch from NULL::timestamp)", "NULL");
+    Ok(())
+}
+
 #[tokio::test]
 async fn test_extract_date_part_func() -> Result<()> {
     test_expression!(
