@@ -19,6 +19,7 @@ use std::error::Error;
 use std::path::{Path, PathBuf};
 
 use log::info;
+use sqllogictest::strict_column_validator;
 
 use datafusion::prelude::{SessionConfig, SessionContext};
 
@@ -67,6 +68,7 @@ async fn run_test_file(
     info!("Running with DataFusion runner: {}", path.display());
     let ctx = context_for_test_file(&relative_path).await;
     let mut runner = sqllogictest::Runner::new(DataFusion::new(ctx, relative_path));
+    runner.with_column_validator(strict_column_validator);
     runner.run_file_async(path).await?;
     Ok(())
 }
@@ -76,13 +78,10 @@ async fn run_test_file_with_postgres(
     relative_path: PathBuf,
 ) -> Result<(), Box<dyn Error>> {
     info!("Running with Postgres runner: {}", path.display());
-
     let postgres_client = Postgres::connect(relative_path).await?;
-
-    sqllogictest::Runner::new(postgres_client)
-        .run_file_async(path)
-        .await?;
-
+    let mut runner = sqllogictest::Runner::new(postgres_client);
+    runner.with_column_validator(strict_column_validator);
+    runner.run_file_async(path).await?;
     Ok(())
 }
 
@@ -90,15 +89,20 @@ async fn run_complete_file(
     path: &Path,
     relative_path: PathBuf,
 ) -> Result<(), Box<dyn Error>> {
-    use sqllogictest::{default_validator, update_test_file};
+    use sqllogictest::default_validator;
 
     info!("Using complete mode to complete: {}", path.display());
 
     let ctx = context_for_test_file(&relative_path).await;
     let mut runner = sqllogictest::Runner::new(DataFusion::new(ctx, relative_path));
     let col_separator = " ";
-    let validator = default_validator;
-    update_test_file(path, &mut runner, col_separator, validator)
+    runner
+        .update_test_file(
+            path,
+            col_separator,
+            default_validator,
+            strict_column_validator,
+        )
         .await
         .map_err(|e| e.to_string())?;
 
