@@ -22,7 +22,7 @@ use crate::expr::{
     Like, Sort, TryCast, WindowFunction,
 };
 use crate::logical_plan::Projection;
-use crate::{Expr, ExprSchemable, LogicalPlan};
+use crate::{Expr, ExprSchemable, LogicalPlan, LogicalPlanBuilder};
 use datafusion_common::Result;
 use datafusion_common::{Column, DFSchema};
 use std::collections::HashMap;
@@ -121,6 +121,7 @@ impl ExprRewritable for Expr {
         let expr = match self {
             Expr::Alias(expr, name) => Expr::Alias(rewrite_boxed(expr, rewriter)?, name),
             Expr::Column(_) => self.clone(),
+            Expr::OuterReferenceColumn(_, _) => self.clone(),
             Expr::Exists { .. } => self.clone(),
             Expr::InSubquery {
                 expr,
@@ -344,11 +345,23 @@ where
 /// Recursively call [`Column::normalize_with_schemas`] on all [`Column`] expressions
 /// in the `expr` expression tree.
 pub fn normalize_col(expr: Expr, plan: &LogicalPlan) -> Result<Expr> {
-    normalize_col_with_schemas(expr, &plan.all_schemas(), &plan.using_columns()?)
+    rewrite_expr(expr, |expr| {
+        if let Expr::Column(c) = expr {
+            let col = LogicalPlanBuilder::normalize(plan, c)?;
+            Ok(Expr::Column(col))
+        } else {
+            Ok(expr)
+        }
+    })
 }
 
 /// Recursively call [`Column::normalize_with_schemas`] on all [`Column`] expressions
 /// in the `expr` expression tree.
+#[deprecated(
+    since = "20.0.0",
+    note = "use normalize_col_with_schemas_and_ambiguity_check instead"
+)]
+#[allow(deprecated)]
 pub fn normalize_col_with_schemas(
     expr: Expr,
     schemas: &[&Arc<DFSchema>],
@@ -365,6 +378,24 @@ pub fn normalize_col_with_schemas(
     })
 }
 
+/// See [`Column::normalize_with_schemas_and_ambiguity_check`] for usage
+pub fn normalize_col_with_schemas_and_ambiguity_check(
+    expr: Expr,
+    schemas: &[&[&DFSchema]],
+    using_columns: &[HashSet<Column>],
+) -> Result<Expr> {
+    rewrite_expr(expr, |expr| {
+        if let Expr::Column(c) = expr {
+            Ok(Expr::Column(c.normalize_with_schemas_and_ambiguity_check(
+                schemas,
+                using_columns,
+            )?))
+        } else {
+            Ok(expr)
+        }
+    })
+}
+
 /// Recursively normalize all [`Column`] expressions in a list of expression trees
 pub fn normalize_cols(
     exprs: impl IntoIterator<Item = impl Into<Expr>>,
@@ -416,6 +447,19 @@ pub fn unnormalize_cols(exprs: impl IntoIterator<Item = Expr>) -> Vec<Expr> {
     exprs.into_iter().map(unnormalize_col).collect()
 }
 
+/// Recursively remove all the ['OuterReferenceColumn'] and return the inside Column
+/// in the expression tree.
+pub fn strip_outer_reference(expr: Expr) -> Expr {
+    rewrite_expr(expr, |expr| {
+        if let Expr::OuterReferenceColumn(_, col) = expr {
+            Ok(Expr::Column(col))
+        } else {
+            Ok(expr)
+        }
+    })
+    .expect("strip_outer_reference is infallable")
+}
+
 /// Implementation of [`ExprRewriter`] that calls a function, for use
 /// with [`rewrite_expr`]
 struct RewriterAdapter<F> {
@@ -600,13 +644,12 @@ mod test {
             make_field("tableC", "f"),
             make_field("tableC", "ff"),
         ]);
-        let schemas = vec![schema_c, schema_f, schema_b, schema_a]
-            .into_iter()
-            .map(Arc::new)
-            .collect::<Vec<_>>();
+        let schemas = vec![schema_c, schema_f, schema_b, schema_a];
         let schemas = schemas.iter().collect::<Vec<_>>();
 
-        let normalized_expr = normalize_col_with_schemas(expr, &schemas, &[]).unwrap();
+        let normalized_expr =
+            normalize_col_with_schemas_and_ambiguity_check(expr, &[&schemas], &[])
+                .unwrap();
         assert_eq!(
             normalized_expr,
             col("tableA.a") + col("tableB.b") + col("tableC.c")
@@ -614,6 +657,7 @@ mod test {
     }
 
     #[test]
+    #[allow(deprecated)]
     fn normalize_cols_priority() {
         let expr = col("a") + col("b");
         // Schemas with multiple matches for column a, first takes priority
@@ -634,16 +678,18 @@ mod test {
     fn normalize_cols_non_exist() {
         // test normalizing columns when the name doesn't exist
         let expr = col("a") + col("b");
-        let schema_a = make_schema_with_empty_metadata(vec![make_field("tableA", "a")]);
-        let schemas = vec![schema_a].into_iter().map(Arc::new).collect::<Vec<_>>();
+        let schema_a =
+            make_schema_with_empty_metadata(vec![make_field("\"tableA\"", "a")]);
+        let schemas = vec![schema_a];
         let schemas = schemas.iter().collect::<Vec<_>>();
 
-        let error = normalize_col_with_schemas(expr, &schemas, &[])
-            .unwrap_err()
-            .to_string();
+        let error =
+            normalize_col_with_schemas_and_ambiguity_check(expr, &[&schemas], &[])
+                .unwrap_err()
+                .to_string();
         assert_eq!(
             error,
-            "Schema error: No field named 'b'. Valid fields are 'tableA'.'a'."
+            r#"Schema error: No field named "b". Valid fields are "tableA"."a"."#
         );
     }
 
@@ -659,7 +705,7 @@ mod test {
     }
 
     fn make_field(relation: &str, column: &str) -> DFField {
-        DFField::new(Some(relation), column, DataType::Int8, false)
+        DFField::new(Some(relation.to_string()), column, DataType::Int8, false)
     }
 
     #[test]
