@@ -24,8 +24,8 @@ use crate::logical_plan::display::{GraphvizVisitor, IndentVisitor};
 use crate::logical_plan::extension::UserDefinedLogicalNode;
 use crate::logical_plan::plan;
 use crate::utils::{
-    self, exprlist_to_fields, from_plan, grouping_set_expr_count,
-    grouping_set_to_exprlist,
+    enumerate_grouping_sets, exprlist_to_fields, find_out_reference_exprs, from_plan,
+    grouping_set_expr_count, grouping_set_to_exprlist,
 };
 use crate::{
     build_join_schema, Expr, ExprSchemable, TableProviderFilterPushDown, TableSource,
@@ -34,7 +34,7 @@ use arrow::datatypes::{DataType, Field, Schema, SchemaRef};
 use datafusion_common::parsers::CompressionTypeVariant;
 use datafusion_common::{
     plan_err, Column, DFSchema, DFSchemaRef, DataFusionError, OwnedTableReference,
-    ScalarValue,
+    Result, ScalarValue, TableReference,
 };
 use std::collections::{HashMap, HashSet};
 use std::fmt::{self, Debug, Display, Formatter};
@@ -174,7 +174,26 @@ impl LogicalPlan {
         }
     }
 
+    /// Used for normalizing columns, as the fallback schemas to the main schema
+    /// of the plan.
+    pub fn fallback_normalize_schemas(&self) -> Vec<&DFSchema> {
+        match self {
+            LogicalPlan::Window(_)
+            | LogicalPlan::Projection(_)
+            | LogicalPlan::Aggregate(_)
+            | LogicalPlan::Unnest(_)
+            | LogicalPlan::Join(_)
+            | LogicalPlan::CrossJoin(_) => self
+                .inputs()
+                .iter()
+                .map(|input| input.schema().as_ref())
+                .collect(),
+            _ => vec![],
+        }
+    }
+
     /// Get all meaningful schemas of a plan and its children plan.
+    #[deprecated(since = "20.0.0")]
     pub fn all_schemas(&self) -> Vec<&DFSchemaRef> {
         match self {
             // return self and children schemas
@@ -240,10 +259,35 @@ impl LogicalPlan {
         let mut exprs = vec![];
         self.inspect_expressions(|e| {
             exprs.push(e.clone());
+            Ok(()) as Result<()>
+        })
+        // closure always returns OK
+        .unwrap();
+        exprs
+    }
+
+    /// Returns all the out reference(correlated) expressions (recursively) in the current
+    /// logical plan nodes and all its descendant nodes.
+    pub fn all_out_ref_exprs(self: &LogicalPlan) -> Vec<Expr> {
+        let mut exprs = vec![];
+        self.inspect_expressions(|e| {
+            find_out_reference_exprs(e).into_iter().for_each(|e| {
+                if !exprs.contains(&e) {
+                    exprs.push(e)
+                }
+            });
             Ok(()) as Result<(), DataFusionError>
         })
         // closure always returns OK
         .unwrap();
+        self.inputs()
+            .into_iter()
+            .flat_map(|child| child.all_out_ref_exprs())
+            .for_each(|e| {
+                if !exprs.contains(&e) {
+                    exprs.push(e)
+                }
+            });
         exprs
     }
 
@@ -373,7 +417,7 @@ impl LogicalPlan {
     }
 
     /// returns all `Using` join columns in a logical plan
-    pub fn using_columns(&self) -> Result<Vec<HashSet<Column>>, DataFusionError> {
+    pub fn using_columns(&self) -> Result<Vec<HashSet<Column>>> {
         struct UsingJoinColumnVisitor {
             using_columns: Vec<HashSet<Column>>,
         }
@@ -393,7 +437,7 @@ impl LogicalPlan {
                         on.iter().try_fold(HashSet::new(), |mut accumu, (l, r)| {
                             accumu.insert(l.try_into_col()?);
                             accumu.insert(r.try_into_col()?);
-                            Result::<_, DataFusionError>::Ok(accumu)
+                            Result::<_>::Ok(accumu)
                         })?;
                     self.using_columns.push(columns);
                 }
@@ -408,10 +452,7 @@ impl LogicalPlan {
         Ok(visitor.using_columns)
     }
 
-    pub fn with_new_inputs(
-        &self,
-        inputs: &[LogicalPlan],
-    ) -> Result<LogicalPlan, DataFusionError> {
+    pub fn with_new_inputs(&self, inputs: &[LogicalPlan]) -> Result<LogicalPlan> {
         from_plan(self, &self.expressions(), inputs)
     }
 
@@ -420,7 +461,7 @@ impl LogicalPlan {
     pub fn with_param_values(
         self,
         param_values: Vec<ScalarValue>,
-    ) -> Result<LogicalPlan, DataFusionError> {
+    ) -> Result<LogicalPlan> {
         match self {
             LogicalPlan::Prepare(prepare_lp) => {
                 // Verify if the number of params matches the number of values
@@ -613,28 +654,25 @@ impl LogicalPlan {
     /// params_values
     pub fn replace_params_with_values(
         &self,
-        param_values: &Vec<ScalarValue>,
-    ) -> Result<LogicalPlan, DataFusionError> {
-        let exprs = self.expressions();
-        let mut new_exprs = vec![];
-        for expr in exprs {
-            new_exprs.push(Self::replace_placeholders_with_values(expr, param_values)?);
-        }
+        param_values: &[ScalarValue],
+    ) -> Result<LogicalPlan> {
+        let new_exprs = self
+            .expressions()
+            .into_iter()
+            .map(|e| Self::replace_placeholders_with_values(e, param_values))
+            .collect::<Result<Vec<_>>>()?;
 
-        let new_inputs = self.inputs();
-        let mut new_inputs_with_values = vec![];
-        for input in new_inputs {
-            new_inputs_with_values.push(input.replace_params_with_values(param_values)?);
-        }
+        let new_inputs_with_values = self
+            .inputs()
+            .into_iter()
+            .map(|inp| inp.replace_params_with_values(param_values))
+            .collect::<Result<Vec<_>>>()?;
 
-        let new_plan = utils::from_plan(self, &new_exprs, &new_inputs_with_values)?;
-        Ok(new_plan)
+        from_plan(self, &new_exprs, &new_inputs_with_values)
     }
 
     /// Walk the logical plan, find any `PlaceHolder` tokens, and return a map of their IDs and DataTypes
-    pub fn get_parameter_types(
-        &self,
-    ) -> Result<HashMap<String, Option<DataType>>, DataFusionError> {
+    pub fn get_parameter_types(&self) -> Result<HashMap<String, Option<DataType>>> {
         struct ParamTypeVisitor {
             param_types: HashMap<String, Option<DataType>>,
         }
@@ -644,10 +682,7 @@ impl LogicalPlan {
         }
 
         impl ExpressionVisitor for ExprParamTypeVisitor {
-            fn pre_visit(
-                mut self,
-                expr: &Expr,
-            ) -> datafusion_common::Result<Recursion<Self>>
+            fn pre_visit(mut self, expr: &Expr) -> Result<Recursion<Self>>
             where
                 Self: ExpressionVisitor,
             {
@@ -682,7 +717,7 @@ impl LogicalPlan {
                     };
                     visitor = expr.accept(visitor)?;
                     param_types.extend(visitor.param_types);
-                    Ok(()) as Result<(), DataFusionError>
+                    Ok(()) as Result<()>
                 })?;
                 self.param_types.extend(param_types);
                 Ok(true)
@@ -701,7 +736,7 @@ impl LogicalPlan {
     fn replace_placeholders_with_values(
         expr: Expr,
         param_values: &[ScalarValue],
-    ) -> Result<Expr, DataFusionError> {
+    ) -> Result<Expr> {
         rewrite_expr(expr, |expr| {
             match &expr {
                 Expr::Placeholder { id, data_type } => {
@@ -729,11 +764,12 @@ impl LogicalPlan {
                     Ok(Expr::Literal(value.clone()))
                 }
                 Expr::ScalarSubquery(qry) => {
-                    let subquery = Arc::new(
-                        qry.subquery
-                            .replace_params_with_values(&param_values.to_vec())?,
-                    );
-                    Ok(Expr::ScalarSubquery(plan::Subquery { subquery }))
+                    let subquery =
+                        Arc::new(qry.subquery.replace_params_with_values(param_values)?);
+                    Ok(Expr::ScalarSubquery(plan::Subquery {
+                        subquery,
+                        outer_ref_columns: qry.outer_ref_columns.clone(),
+                    }))
                 }
                 _ => Ok(expr),
             }
@@ -1349,10 +1385,7 @@ pub struct Projection {
 
 impl Projection {
     /// Create a new Projection
-    pub fn try_new(
-        expr: Vec<Expr>,
-        input: Arc<LogicalPlan>,
-    ) -> Result<Self, DataFusionError> {
+    pub fn try_new(expr: Vec<Expr>, input: Arc<LogicalPlan>) -> Result<Self> {
         let schema = Arc::new(DFSchema::new_with_metadata(
             exprlist_to_fields(&expr, &input)?,
             input.schema().metadata().clone(),
@@ -1365,7 +1398,7 @@ impl Projection {
         expr: Vec<Expr>,
         input: Arc<LogicalPlan>,
         schema: DFSchemaRef,
-    ) -> Result<Self, DataFusionError> {
+    ) -> Result<Self> {
         if expr.len() != schema.fields().len() {
             return Err(DataFusionError::Plan(format!("Projection has mismatch between number of expressions ({}) and number of fields in schema ({})", expr.len(), schema.fields().len())));
         }
@@ -1391,7 +1424,7 @@ impl Projection {
         }
     }
 
-    pub fn try_from_plan(plan: &LogicalPlan) -> datafusion_common::Result<&Projection> {
+    pub fn try_from_plan(plan: &LogicalPlan) -> Result<&Projection> {
         match plan {
             LogicalPlan::Projection(it) => Ok(it),
             _ => plan_err!("Could not coerce into Projection!"),
@@ -1413,14 +1446,14 @@ pub struct SubqueryAlias {
 }
 
 impl SubqueryAlias {
-    pub fn try_new(
-        plan: LogicalPlan,
-        alias: impl Into<String>,
-    ) -> datafusion_common::Result<Self> {
+    pub fn try_new(plan: LogicalPlan, alias: impl Into<String>) -> Result<Self> {
         let alias = alias.into();
+        let table_ref = TableReference::bare(&alias);
         let schema: Schema = plan.schema().as_ref().clone().into();
-        let schema =
-            DFSchemaRef::new(DFSchema::try_from_qualified_schema(&alias, &schema)?);
+        let schema = DFSchemaRef::new(DFSchema::try_from_qualified_schema(
+            table_ref.to_owned_reference(),
+            &schema,
+        )?);
         Ok(SubqueryAlias {
             input: Arc::new(plan),
             alias,
@@ -1451,10 +1484,7 @@ pub struct Filter {
 
 impl Filter {
     /// Create a new filter operator.
-    pub fn try_new(
-        predicate: Expr,
-        input: Arc<LogicalPlan>,
-    ) -> datafusion_common::Result<Self> {
+    pub fn try_new(predicate: Expr, input: Arc<LogicalPlan>) -> Result<Self> {
         // Filter predicates must return a boolean value so we try and validate that here.
         // Note that it is not always possible to resolve the predicate expression during plan
         // construction (such as with correlated subqueries) so we make a best effort here and
@@ -1479,7 +1509,7 @@ impl Filter {
         Ok(Self { predicate, input })
     }
 
-    pub fn try_from_plan(plan: &LogicalPlan) -> datafusion_common::Result<&Filter> {
+    pub fn try_from_plan(plan: &LogicalPlan) -> Result<&Filter> {
         match plan {
             LogicalPlan::Filter(it) => Ok(it),
             _ => plan_err!("Could not coerce into Filter!"),
@@ -1502,7 +1532,7 @@ pub struct Window {
 #[derive(Clone)]
 pub struct TableScan {
     /// The name of the table
-    pub table_name: String,
+    pub table_name: OwnedTableReference,
     /// The source of the table
     pub source: Arc<dyn TableSource>,
     /// Optional column indices to use as a projection
@@ -1613,6 +1643,8 @@ pub struct CreateExternalTable {
     pub if_not_exists: bool,
     /// SQL used to create the table, if available
     pub definition: Option<String>,
+    /// Order expressions supplied by user
+    pub order_exprs: Vec<Expr>,
     /// File compression type (GZIP, BZIP2, XZ, ZSTD)
     pub file_compression_type: CompressionTypeVariant,
     /// Table(provider) specific options
@@ -1633,6 +1665,7 @@ impl Hash for CreateExternalTable {
         self.if_not_exists.hash(state);
         self.definition.hash(state);
         self.file_compression_type.hash(state);
+        self.order_exprs.hash(state);
         self.options.len().hash(state); // HashMap is not hashable
     }
 }
@@ -1776,7 +1809,8 @@ impl Aggregate {
         input: Arc<LogicalPlan>,
         group_expr: Vec<Expr>,
         aggr_expr: Vec<Expr>,
-    ) -> datafusion_common::Result<Self> {
+    ) -> Result<Self> {
+        let group_expr = enumerate_grouping_sets(group_expr)?;
         let grouping_expr: Vec<Expr> = grouping_set_to_exprlist(group_expr.as_slice())?;
         let all_expr = grouping_expr.iter().chain(aggr_expr.iter());
         validate_unique_names("Aggregations", all_expr.clone())?;
@@ -1797,7 +1831,7 @@ impl Aggregate {
         group_expr: Vec<Expr>,
         aggr_expr: Vec<Expr>,
         schema: DFSchemaRef,
-    ) -> datafusion_common::Result<Self> {
+    ) -> Result<Self> {
         if group_expr.is_empty() && aggr_expr.is_empty() {
             return Err(DataFusionError::Plan(
                 "Aggregate requires at least one grouping or aggregate expression"
@@ -1820,7 +1854,7 @@ impl Aggregate {
         })
     }
 
-    pub fn try_from_plan(plan: &LogicalPlan) -> datafusion_common::Result<&Aggregate> {
+    pub fn try_from_plan(plan: &LogicalPlan) -> Result<&Aggregate> {
         match plan {
             LogicalPlan::Aggregate(it) => Ok(it),
             _ => plan_err!("Could not coerce into Aggregate!"),
@@ -1867,7 +1901,7 @@ impl Join {
         left: Arc<LogicalPlan>,
         right: Arc<LogicalPlan>,
         column_on: (Vec<Column>, Vec<Column>),
-    ) -> Result<Self, DataFusionError> {
+    ) -> Result<Self> {
         let original_join = match original {
             LogicalPlan::Join(join) => join,
             _ => return plan_err!("Could not create join with project input"),
@@ -1900,21 +1934,25 @@ impl Join {
 pub struct Subquery {
     /// The subquery
     pub subquery: Arc<LogicalPlan>,
+    /// The outer references used in the subquery
+    pub outer_ref_columns: Vec<Expr>,
 }
 
 impl Subquery {
-    pub fn new(plan: LogicalPlan) -> Self {
-        Subquery {
-            subquery: Arc::new(plan),
-        }
-    }
-    pub fn try_from_expr(plan: &Expr) -> datafusion_common::Result<&Subquery> {
+    pub fn try_from_expr(plan: &Expr) -> Result<&Subquery> {
         match plan {
             Expr::ScalarSubquery(it) => Ok(it),
             Expr::Cast(cast) => Subquery::try_from_expr(cast.expr.as_ref()),
             _ => plan_err!("Could not coerce into ScalarSubquery!"),
         }
     }
+
+    pub fn with_plan(&self, plan: Arc<LogicalPlan>) -> Subquery {
+        Subquery {
+            subquery: plan,
+            outer_ref_columns: self.outer_ref_columns.clone(),
+        }
+    }
 }
 
 impl Debug for Subquery {
@@ -2030,7 +2068,6 @@ mod tests {
     use crate::{col, exists, in_subquery, lit};
     use arrow::datatypes::{DataType, Field, Schema};
     use datafusion_common::DFSchema;
-    use datafusion_common::Result;
     use std::collections::HashMap;
 
     fn employee_schema() -> Schema {
@@ -2372,7 +2409,7 @@ mod tests {
             Field::new("state", DataType::Utf8, false),
         ]);
 
-        table_scan(None, &schema, Some(vec![0, 1]))
+        table_scan(TableReference::none(), &schema, Some(vec![0, 1]))
             .unwrap()
             .filter(col("state").eq(lit("CO")))
             .unwrap()
@@ -2439,6 +2476,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(deprecated)]
     fn test_extension_all_schemas() {
         let plan = LogicalPlan::Extension(Extension {
             node: Arc::new(NoChildExtension::empty()),
