@@ -17,9 +17,12 @@
 
 use crate::alias::AliasGenerator;
 use crate::optimizer::ApplyOrder;
-use crate::utils::{conjunction, extract_join_filters, only_or_err, split_conjunction};
+use crate::utils::{
+    collect_subquery_cols, conjunction, extract_join_filters, only_or_err,
+    split_conjunction,
+};
 use crate::{OptimizerConfig, OptimizerRule};
-use datafusion_common::{context, Column, DataFusionError, Result};
+use datafusion_common::{context, Column, Result};
 use datafusion_expr::expr_rewriter::{replace_col, unnormalize_col};
 use datafusion_expr::logical_plan::{JoinType, Projection, Subquery};
 use datafusion_expr::{Expr, Filter, LogicalPlan, LogicalPlanBuilder};
@@ -50,7 +53,7 @@ impl DecorrelateWhereIn {
         &self,
         predicate: &Expr,
         config: &dyn OptimizerConfig,
-    ) -> datafusion_common::Result<(Vec<SubqueryInfo>, Vec<Expr>)> {
+    ) -> Result<(Vec<SubqueryInfo>, Vec<Expr>)> {
         let filters = split_conjunction(predicate); // TODO: disjunctions
 
         let mut subqueries = vec![];
@@ -62,14 +65,16 @@ impl DecorrelateWhereIn {
                     subquery,
                     negated,
                 } => {
-                    let subquery = self
+                    let subquery_plan = self
                         .try_optimize(&subquery.subquery, config)?
                         .map(Arc::new)
                         .unwrap_or_else(|| subquery.subquery.clone());
-                    let subquery = Subquery { subquery };
-                    let subquery =
-                        SubqueryInfo::new(subquery.clone(), (**expr).clone(), *negated);
-                    subqueries.push(subquery);
+                    let new_subquery = subquery.with_plan(subquery_plan);
+                    subqueries.push(SubqueryInfo::new(
+                        new_subquery,
+                        (**expr).clone(),
+                        *negated,
+                    ));
                     // TODO: if subquery doesn't get optimized, optimized children are lost
                 }
                 _ => others.push((*it).clone()),
@@ -146,6 +151,7 @@ fn optimize_where_in(
     let projection = Projection::try_from_plan(&query_info.query.subquery)
         .map_err(|e| context!("a projection is required", e))?;
     let subquery_input = projection.input.clone();
+    // TODO add the validate logic to Analyzer
     let subquery_expr = only_or_err(projection.expr.as_slice())
         .map_err(|e| context!("single expression projection required", e))?;
 
@@ -159,19 +165,7 @@ fn optimize_where_in(
     // replace qualified name with subquery alias.
     let subquery_alias = alias.next("__correlated_sq");
     let input_schema = subquery_input.schema();
-    let mut subquery_cols: BTreeSet<Column> =
-        join_filters
-            .iter()
-            .try_fold(BTreeSet::new(), |mut cols, expr| {
-                let using_cols: Vec<Column> = expr
-                    .to_columns()?
-                    .into_iter()
-                    .filter(|col| input_schema.has_column(col))
-                    .collect::<_>();
-
-                cols.extend(using_cols);
-                Result::<_, DataFusionError>::Ok(cols)
-            })?;
+    let mut subquery_cols = collect_subquery_cols(&join_filters, input_schema.clone())?;
     let join_filter = conjunction(join_filters).map_or(Ok(None), |filter| {
         replace_qualified_name(filter, &subquery_cols, &subquery_alias).map(Option::Some)
     })?;
