@@ -72,12 +72,12 @@ impl ScalarSubqueryToJoin {
                                 Ok(subquery) => subquery,
                                 _ => return Ok(()),
                             };
-                            let subquery = self
+                            let subquery_plan = self
                                 .try_optimize(&subquery.subquery, config)?
                                 .map(Arc::new)
                                 .unwrap_or_else(|| subquery.subquery.clone());
-                            let subquery = Subquery { subquery };
-                            let res = SubqueryInfo::new(subquery, expr, *op, lhs);
+                            let new_subquery = subquery.with_plan(subquery_plan);
+                            let res = SubqueryInfo::new(new_subquery, expr, *op, lhs);
                             subqueries.push(res);
                             Ok(())
                         };
@@ -272,16 +272,10 @@ fn optimize_scalar(
     // qualify the join columns for outside the subquery
     let mut subqry_cols: Vec<_> = subqry_cols
         .iter()
-        .map(|it| Column {
-            relation: Some(subqry_alias.clone()),
-            name: it.name.clone(),
-        })
+        .map(|it| Column::new(Some(subqry_alias.clone()), it.name.clone()))
         .collect();
 
-    let qry_expr = Expr::Column(Column {
-        relation: Some(subqry_alias),
-        name: "__value".to_string(),
-    });
+    let qry_expr = Expr::Column(Column::new(Some(subqry_alias), "__value".to_string()));
 
     // if correlated subquery's operation is column equality, put the clause into join on clause.
     let mut restore_where_clause = true;
@@ -330,7 +324,6 @@ fn optimize_scalar(
         new_plan = new_plan.filter(expr)?
     }
     let new_plan = new_plan.build()?;
-
     Ok(Some(new_plan))
 }
 
