@@ -20,7 +20,7 @@
 use datafusion_common::{DataFusionError, Result, ScalarValue};
 use datafusion_expr::{
     expr::{Between, BinaryExpr},
-    expr_fn::{and, concat_ws, or},
+    expr_fn::{and, bitwise_and, bitwise_or, concat_ws, or},
     lit, BuiltinScalarFunction, Expr, Like, Operator,
 };
 
@@ -311,6 +311,45 @@ pub fn negate_clause(expr: Expr) -> Expr {
     }
 }
 
+/// bitwise negate a Negative clause
+/// input is the clause to be bitwise negated.(args for Negative clause)
+/// For BinaryExpr:
+///    ~(A & B) ===> ~A | ~B
+///    ~(A | B) ===> ~A & ~B
+/// For Negative:
+///    ~(~A) ===> A
+/// For others, use Negative clause
+pub fn distribute_negation(expr: Expr) -> Expr {
+    match expr {
+        Expr::BinaryExpr(BinaryExpr { left, op, right }) => {
+            match op {
+                // ~(A & B) ===> ~A | ~B
+                Operator::BitwiseAnd => {
+                    let left = distribute_negation(*left);
+                    let right = distribute_negation(*right);
+
+                    bitwise_or(left, right)
+                }
+                // ~(A | B) ===> ~A & ~B
+                Operator::BitwiseOr => {
+                    let left = distribute_negation(*left);
+                    let right = distribute_negation(*right);
+
+                    bitwise_and(left, right)
+                }
+                // use negative clause
+                _ => Expr::Negative(Box::new(Expr::BinaryExpr(BinaryExpr::new(
+                    left, op, right,
+                )))),
+            }
+        }
+        // ~(~A) ===> A
+        Expr::Negative(expr) => *expr,
+        // use negative clause
+        _ => Expr::Negative(Box::new(expr)),
+    }
+}
+
 /// Simplify the `concat` function by
 /// 1. filtering out all `null` literals
 /// 2. concatenating contiguous literal arguments
