@@ -21,7 +21,9 @@ use crate::{OptimizerConfig, OptimizerRule};
 use datafusion_common::{plan_err, Column, DFSchemaRef};
 use datafusion_common::{DFSchema, Result};
 use datafusion_expr::expr::{BinaryExpr, Sort};
-use datafusion_expr::expr_rewriter::{ExprRewritable, ExprRewriter};
+use datafusion_expr::expr_rewriter::{
+    strip_outer_reference, ExprRewritable, ExprRewriter,
+};
 use datafusion_expr::expr_visitor::inspect_expr_pre;
 use datafusion_expr::logical_plan::LogicalPlanBuilder;
 use datafusion_expr::utils::{check_all_columns_from_schema, from_plan};
@@ -30,7 +32,7 @@ use datafusion_expr::{
     logical_plan::{Filter, LogicalPlan},
     Expr, Operator,
 };
-use std::collections::HashSet;
+use std::collections::{BTreeSet, HashSet};
 use std::sync::Arc;
 
 /// Convenience rule for writing optimizers: recursively invoke
@@ -291,46 +293,52 @@ pub fn find_join_exprs(
     let mut joins = vec![];
     let mut others = vec![];
     for filter in exprs.iter() {
-        let (left, op, right) = match filter {
-            Expr::BinaryExpr(BinaryExpr { left, op, right }) => {
-                (*left.clone(), *op, *right.clone())
-            }
-            _ => {
+        // If the expression contains correlated predicates, add it to join filters
+        if filter.contains_outer() {
+            joins.push(strip_outer_reference((*filter).clone()));
+        } else {
+            // TODO remove the logic
+            let (left, op, right) = match filter {
+                Expr::BinaryExpr(BinaryExpr { left, op, right }) => {
+                    (*left.clone(), *op, *right.clone())
+                }
+                _ => {
+                    others.push((*filter).clone());
+                    continue;
+                }
+            };
+            let left = match left {
+                Expr::Column(c) => c,
+                _ => {
+                    others.push((*filter).clone());
+                    continue;
+                }
+            };
+            let right = match right {
+                Expr::Column(c) => c,
+                _ => {
+                    others.push((*filter).clone());
+                    continue;
+                }
+            };
+            if fields.contains(&left.flat_name()) && fields.contains(&right.flat_name()) {
                 others.push((*filter).clone());
-                continue;
+                continue; // both columns present (none closed-upon)
             }
-        };
-        let left = match left {
-            Expr::Column(c) => c,
-            _ => {
+            if !fields.contains(&left.flat_name()) && !fields.contains(&right.flat_name())
+            {
                 others.push((*filter).clone());
-                continue;
+                continue; // neither column present (syntax error?)
             }
-        };
-        let right = match right {
-            Expr::Column(c) => c,
-            _ => {
-                others.push((*filter).clone());
-                continue;
+            match op {
+                Operator::Eq => {}
+                Operator::NotEq => {}
+                _ => {
+                    plan_err!(format!("can't optimize {op} column comparison"))?;
+                }
             }
-        };
-        if fields.contains(&left.flat_name()) && fields.contains(&right.flat_name()) {
-            others.push((*filter).clone());
-            continue; // both columns present (none closed-upon)
+            joins.push((*filter).clone())
         }
-        if !fields.contains(&left.flat_name()) && !fields.contains(&right.flat_name()) {
-            others.push((*filter).clone());
-            continue; // neither column present (syntax error?)
-        }
-        match op {
-            Operator::Eq => {}
-            Operator::NotEq => {}
-            _ => {
-                plan_err!(format!("can't optimize {op} column comparison"))?;
-            }
-        }
-
-        joins.push((*filter).clone())
     }
 
     Ok((joins, others))
@@ -460,13 +468,17 @@ fn add_alias_if_changed(original_name: String, expr: Expr) -> Result<Expr> {
 
 /// merge inputs schema into a single schema.
 pub fn merge_schema(inputs: Vec<&LogicalPlan>) -> DFSchema {
-    inputs
-        .iter()
-        .map(|input| input.schema())
-        .fold(DFSchema::empty(), |mut lhs, rhs| {
-            lhs.merge(rhs);
-            lhs
-        })
+    if inputs.len() == 1 {
+        inputs[0].schema().clone().as_ref().clone()
+    } else {
+        inputs.iter().map(|input| input.schema()).fold(
+            DFSchema::empty(),
+            |mut lhs, rhs| {
+                lhs.merge(rhs);
+                lhs
+            },
+        )
+    }
 }
 
 /// Extract join predicates from the correclated subquery.
@@ -485,11 +497,16 @@ pub(crate) fn extract_join_filters(
         let mut join_filters: Vec<Expr> = vec![];
         let mut subquery_filters: Vec<Expr> = vec![];
         for expr in subquery_filter_exprs {
-            let cols = expr.to_columns()?;
-            if check_all_columns_from_schema(&cols, input_schema.clone())? {
-                subquery_filters.push(expr.clone());
+            // If the expression contains correlated predicates, add it to join filters
+            if expr.contains_outer() {
+                join_filters.push(strip_outer_reference(expr.clone()))
             } else {
-                join_filters.push(expr.clone())
+                let cols = expr.to_columns()?;
+                if check_all_columns_from_schema(&cols, input_schema.clone())? {
+                    subquery_filters.push(expr.clone());
+                } else {
+                    join_filters.push(expr.clone())
+                }
             }
         }
 
@@ -505,6 +522,23 @@ pub(crate) fn extract_join_filters(
     }
 }
 
+pub(crate) fn collect_subquery_cols(
+    exprs: &[Expr],
+    subquery_schema: DFSchemaRef,
+) -> Result<BTreeSet<Column>> {
+    exprs.iter().try_fold(BTreeSet::new(), |mut cols, expr| {
+        let mut using_cols: Vec<Column> = vec![];
+        for col in expr.to_columns()?.into_iter() {
+            if subquery_schema.has_column(&col) {
+                using_cols.push(col);
+            }
+        }
+
+        cols.extend(using_cols);
+        Result::<_>::Ok(cols)
+    })
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
