@@ -26,6 +26,7 @@ pub mod execution_props;
 pub mod expressions;
 pub mod functions;
 pub mod hash_utils;
+pub mod intervals;
 pub mod math_expressions;
 mod physical_expr;
 pub mod planner;
@@ -49,7 +50,7 @@ pub use aggregate::AggregateExpr;
 pub use datafusion_common::from_slice;
 pub use equivalence::EquivalenceProperties;
 pub use equivalence::EquivalentClass;
-pub use physical_expr::{AnalysisContext, ExprBoundaries, PhysicalExpr};
+pub use physical_expr::{AnalysisContext, ExprBoundaries, PhysicalExpr, PhysicalExprRef};
 pub use planner::create_physical_expr;
 pub use scalar_function::ScalarFunctionExpr;
 pub use sort_expr::PhysicalSortExpr;
