@@ -32,13 +32,13 @@ impl FunctionRegistry for NoRegistry {
 
     fn udf(&self, name: &str) -> Result<Arc<ScalarUDF>> {
         Err(DataFusionError::Plan(
-            format!("No function registry provided to deserialize, so can not deserialize User Defined Function '{}'", name))
+            format!("No function registry provided to deserialize, so can not deserialize User Defined Function '{name}'"))
         )
     }
 
     fn udaf(&self, name: &str) -> Result<Arc<AggregateUDF>> {
         Err(DataFusionError::Plan(
-            format!("No function registry provided to deserialize, so can not deserialize User Defined Aggregate Function '{}'", name))
+            format!("No function registry provided to deserialize, so can not deserialize User Defined Aggregate Function '{name}'"))
         )
     }
 }
