@@ -21,13 +21,14 @@ use std::sync::Arc;
 use std::vec;
 
 use arrow_schema::*;
+use datafusion_common::field_not_found;
 use sqlparser::ast::ExactNumberInfo;
 use sqlparser::ast::TimezoneInfo;
 use sqlparser::ast::{ColumnDef as SQLColumnDef, ColumnOption};
 use sqlparser::ast::{DataType as SQLDataType, Ident, ObjectName, TableAlias};
 
 use datafusion_common::config::ConfigOptions;
-use datafusion_common::{field_not_found, DFSchema, DataFusionError, Result};
+use datafusion_common::{unqualified_field_not_found, DFSchema, DataFusionError, Result};
 use datafusion_common::{OwnedTableReference, TableReference};
 use datafusion_expr::logical_plan::{LogicalPlan, LogicalPlanBuilder};
 use datafusion_expr::utils::find_column_exprs;
@@ -69,13 +70,18 @@ impl Default for ParserOptions {
 }
 
 #[derive(Debug, Clone)]
-/// Struct to store Common Table Expression (CTE) provided with WITH clause and
-/// Parameter Data Types provided with PREPARE statement
+/// Struct to store the states used by the Planner. The Planner will leverage the states to resolve
+/// CTEs, Views, subqueries and PREPARE statements. The states include
+/// Common Table Expression (CTE) provided with WITH clause and
+/// Parameter Data Types provided with PREPARE statement and the query schema of the
+/// outer query plan
 pub struct PlannerContext {
     /// Data type provided with prepare statement
     pub prepare_param_data_types: Vec<DataType>,
     /// Map of CTE name to logical plan of the WITH clause
     pub ctes: HashMap<String, LogicalPlan>,
+    /// The query schema of the outer query plan, used to resolve the columns in subquery
+    pub outer_query_schema: Option<DFSchema>,
 }
 
 impl Default for PlannerContext {
@@ -90,6 +96,7 @@ impl PlannerContext {
         Self {
             prepare_param_data_types: vec![],
             ctes: HashMap::new(),
+            outer_query_schema: None,
         }
     }
 
@@ -100,6 +107,7 @@ impl PlannerContext {
         Self {
             prepare_param_data_types,
             ctes: HashMap::new(),
+            outer_query_schema: None,
         }
     }
 }
@@ -129,14 +137,14 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
         for column in columns {
             let data_type = self.convert_simple_data_type(&column.data_type)?;
-            let allow_null = column
+            let not_nullable = column
                 .options
                 .iter()
-                .any(|x| x.option == ColumnOption::Null);
+                .any(|x| x.option == ColumnOption::NotNull);
             fields.push(Field::new(
                 normalize_ident(column.name, self.options.enable_ident_normalization),
                 data_type,
-                allow_null,
+                !not_nullable,
             ));
         }
 
@@ -201,16 +209,12 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                         if !schema.fields_with_unqualified_name(&col.name).is_empty() {
                             Ok(())
                         } else {
-                            Err(field_not_found(None, col.name.as_str(), schema))
+                            Err(unqualified_field_not_found(col.name.as_str(), schema))
                         }
                     }
                 }
                 .map_err(|_: DataFusionError| {
-                    field_not_found(
-                        col.relation.as_ref().map(|s| s.to_owned()),
-                        col.name.as_str(),
-                        schema,
-                    )
+                    field_not_found(col.relation.clone(), col.name.as_str(), schema)
                 }),
                 _ => Err(DataFusionError::Internal("Not a column".to_string())),
             })
@@ -377,22 +381,18 @@ pub(crate) fn idents_to_table_reference(
     match taker.0.len() {
         1 => {
             let table = taker.take(enable_normalization);
-            Ok(OwnedTableReference::Bare { table })
+            Ok(OwnedTableReference::bare(table))
         }
         2 => {
             let table = taker.take(enable_normalization);
             let schema = taker.take(enable_normalization);
-            Ok(OwnedTableReference::Partial { schema, table })
+            Ok(OwnedTableReference::partial(schema, table))
         }
         3 => {
             let table = taker.take(enable_normalization);
             let schema = taker.take(enable_normalization);
             let catalog = taker.take(enable_normalization);
-            Ok(OwnedTableReference::Full {
-                catalog,
-                schema,
-                table,
-            })
+            Ok(OwnedTableReference::full(catalog, schema, table))
         }
         _ => Err(DataFusionError::Plan(format!(
             "Unsupported compound identifier '{:?}'",
