@@ -20,14 +20,14 @@ use crate::utils::{
     check_columns_satisfy_exprs, extract_aliases, normalize_ident, rebase_expr,
     resolve_aliases_to_exprs, resolve_columns, resolve_positions_to_exprs,
 };
-use datafusion_common::{Column, DFSchema, DFSchemaRef, DataFusionError, Result};
-use datafusion_expr::expr_rewriter::{normalize_col, normalize_col_with_schemas};
+use datafusion_common::{DataFusionError, Result};
+use datafusion_expr::expr_rewriter::{
+    normalize_col, normalize_col_with_schemas_and_ambiguity_check,
+};
 use datafusion_expr::logical_plan::builder::project;
-use datafusion_expr::logical_plan::Join as HashJoin;
-use datafusion_expr::logical_plan::JoinConstraint as HashJoinConstraint;
 use datafusion_expr::utils::{
     expand_qualified_wildcard, expand_wildcard, expr_as_column_expr, expr_to_columns,
-    find_aggregate_exprs, find_column_exprs, find_window_exprs,
+    find_aggregate_exprs, find_window_exprs,
 };
 use datafusion_expr::Expr::Alias;
 use datafusion_expr::{
@@ -44,7 +44,6 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         &self,
         select: Select,
         planner_context: &mut PlannerContext,
-        outer_query_schema: Option<&DFSchema>,
     ) -> Result<LogicalPlan> {
         // check for unsupported syntax first
         if !select.cluster_by.is_empty() {
@@ -66,18 +65,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         // process `from` clause
         let plan = self.plan_from_tables(select.from, planner_context)?;
         let empty_from = matches!(plan, LogicalPlan::EmptyRelation(_));
-        // build from schema for unqualifier column ambiguous check
-        // we should get only one field for unqualifier column from schema.
-        let from_schema = self.build_schema_for_ambiguous_check(&plan)?;
 
         // process `where` clause
-        let plan = self.plan_selection(
-            select.selection,
-            plan,
-            outer_query_schema,
-            planner_context,
-            &from_schema,
-        )?;
+        let plan = self.plan_selection(select.selection, plan, planner_context)?;
 
         // process the SELECT expressions, with wildcards expanded.
         let select_exprs = self.prepare_select_exprs(
@@ -85,7 +75,6 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             select.projection,
             empty_from,
             planner_context,
-            &from_schema,
         )?;
 
         // having and group by clause may reference aliases defined in select projection
@@ -239,35 +228,26 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         &self,
         selection: Option<SQLExpr>,
         plan: LogicalPlan,
-        outer_query_schema: Option<&DFSchema>,
         planner_context: &mut PlannerContext,
-        from_schema: &DFSchema,
     ) -> Result<LogicalPlan> {
         match selection {
             Some(predicate_expr) => {
-                let mut join_schema = (**plan.schema()).clone();
-                let mut all_schemas: Vec<DFSchemaRef> = vec![];
-                for schema in plan.all_schemas() {
-                    all_schemas.push(schema.clone());
-                }
-                if let Some(outer) = outer_query_schema {
-                    all_schemas.push(Arc::new(outer.clone()));
-                    join_schema.merge(outer);
-                }
-                let x: Vec<&DFSchemaRef> = all_schemas.iter().collect();
+                let fallback_schemas = plan.fallback_normalize_schemas();
+                let outer_query_schema = planner_context.outer_query_schema.clone();
+                let outer_query_schema_vec =
+                    if let Some(outer) = outer_query_schema.as_ref() {
+                        vec![outer]
+                    } else {
+                        vec![]
+                    };
 
                 let filter_expr =
-                    self.sql_to_expr(predicate_expr, &join_schema, planner_context)?;
-                self.column_reference_ambiguous_check(
-                    &[filter_expr.clone()],
-                    from_schema,
-                    outer_query_schema,
-                )?;
+                    self.sql_to_expr(predicate_expr, plan.schema(), planner_context)?;
                 let mut using_columns = HashSet::new();
                 expr_to_columns(&filter_expr, &mut using_columns)?;
-                let filter_expr = normalize_col_with_schemas(
+                let filter_expr = normalize_col_with_schemas_and_ambiguity_check(
                     filter_expr,
-                    x.as_slice(),
+                    &[&[plan.schema()], &fallback_schemas, &outer_query_schema_vec],
                     &[using_columns],
                 )?;
 
@@ -315,19 +295,10 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         projection: Vec<SelectItem>,
         empty_from: bool,
         planner_context: &mut PlannerContext,
-        from_schema: &DFSchema,
     ) -> Result<Vec<Expr>> {
         projection
             .into_iter()
-            .map(|expr| {
-                self.sql_select_to_rex(
-                    expr,
-                    plan,
-                    empty_from,
-                    planner_context,
-                    from_schema,
-                )
-            })
+            .map(|expr| self.sql_select_to_rex(expr, plan, empty_from, planner_context))
             .flat_map(|result| match result {
                 Ok(vec) => vec.into_iter().map(Ok).collect(),
                 Err(err) => vec![Err(err)],
@@ -342,28 +313,27 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         plan: &LogicalPlan,
         empty_from: bool,
         planner_context: &mut PlannerContext,
-        from_schema: &DFSchema,
     ) -> Result<Vec<Expr>> {
         match sql {
             SelectItem::UnnamedExpr(expr) => {
                 let expr = self.sql_to_expr(expr, plan.schema(), planner_context)?;
-                self.column_reference_ambiguous_check(
-                    &[expr.clone()],
-                    from_schema,
-                    None,
+                let col = normalize_col_with_schemas_and_ambiguity_check(
+                    expr,
+                    &[&[plan.schema()]],
+                    &plan.using_columns()?,
                 )?;
-                Ok(vec![normalize_col(expr, plan)?])
+                Ok(vec![col])
             }
             SelectItem::ExprWithAlias { expr, alias } => {
                 let select_expr =
                     self.sql_to_expr(expr, plan.schema(), planner_context)?;
-                self.column_reference_ambiguous_check(
-                    &[select_expr.clone()],
-                    from_schema,
-                    None,
+                let col = normalize_col_with_schemas_and_ambiguity_check(
+                    select_expr,
+                    &[&[plan.schema()]],
+                    &plan.using_columns()?,
                 )?;
-                let expr = Alias(Box::new(select_expr), normalize_ident(alias));
-                Ok(vec![normalize_col(expr, plan)?])
+                let expr = Alias(Box::new(col), normalize_ident(alias));
+                Ok(vec![expr])
             }
             SelectItem::Wildcard(options) => {
                 Self::check_wildcard_options(options)?;
@@ -386,49 +356,6 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
     }
 
-    /// ambiguous check for unqualifier column
-    fn column_reference_ambiguous_check(
-        &self,
-        exprs: &[Expr],
-        schema: &DFSchema,
-        fallback_schema: Option<&DFSchema>,
-    ) -> Result<()> {
-        find_column_exprs(exprs)
-            .iter()
-            .try_for_each(|col| match col {
-                Expr::Column(Column {
-                    name,
-                    relation: None,
-                    ..
-                }) => {
-                    match (
-                        schema.fields_with_unqualified_name(name).len(),
-                        fallback_schema,
-                    ) {
-                        // in case is from outer query if this is inner subquery
-                        (0, Some(fallback_schema)) => {
-                            match fallback_schema.fields_with_unqualified_name(name).len()
-                            {
-                                0 => Err(DataFusionError::Plan(format!(
-                                    "column reference {name} is unknown",
-                                ))),
-                                1 => Ok(()),
-                                _ => Err(DataFusionError::Plan(format!(
-                                    "column reference {name} is ambiguous",
-                                ))),
-                            }
-                        }
-                        (1, _) => Ok(()),
-                        // should get only one field in from_schema.
-                        _ => Err(DataFusionError::Plan(format!(
-                            "column reference {name} is ambiguous",
-                        ))),
-                    }
-                }
-                _ => Ok(()),
-            })
-    }
-
     fn check_wildcard_options(options: WildcardAdditionalOptions) -> Result<()> {
         let WildcardAdditionalOptions {
             opt_exclude,
@@ -451,34 +378,6 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
     }
 
-    /// build schema for unqualifier column ambiguous check
-    fn build_schema_for_ambiguous_check(&self, plan: &LogicalPlan) -> Result<DFSchema> {
-        let mut fields = plan.schema().fields().clone();
-
-        let metadata = plan.schema().metadata().clone();
-        if let LogicalPlan::Join(HashJoin {
-            join_constraint: HashJoinConstraint::Using,
-            ref on,
-            ref left,
-            ..
-        }) = plan
-        {
-            // For query: select id from t1 join t2 using(id), this is legal.
-            // We should dedup the fields for cols in using clause.
-            for join_keys in on.iter() {
-                let join_col = &join_keys.0.try_into_col()?;
-                let left_field = left.schema().field_from_column(join_col)?;
-                fields.retain(|field| {
-                    field.unqualified_column().name
-                        != left_field.unqualified_column().name
-                });
-                fields.push(left_field.clone());
-            }
-        }
-
-        DFSchema::new_with_metadata(fields, metadata)
-    }
-
     /// Wrap a plan in a projection
     fn project(&self, input: LogicalPlan, expr: Vec<Expr>) -> Result<LogicalPlan> {
         self.validate_schema_satisfies_exprs(input.schema(), &expr)?;
