@@ -28,22 +28,24 @@ use datafusion_common::{
     Column, DFField, DFSchema, DFSchemaRef, DataFusionError, ExprSchema,
     OwnedTableReference, Result, TableReference, ToDFSchema,
 };
-use datafusion_expr::expr_rewriter::normalize_col_with_schemas;
+use datafusion_expr::expr_rewriter::normalize_col_with_schemas_and_ambiguity_check;
 use datafusion_expr::logical_plan::builder::project;
 use datafusion_expr::logical_plan::{Analyze, Prepare};
 use datafusion_expr::utils::expr_to_columns;
 use datafusion_expr::{
     cast, col, CreateCatalog, CreateCatalogSchema,
     CreateExternalTable as PlanCreateExternalTable, CreateMemoryTable, CreateView,
-    DescribeTable, DmlStatement, DropTable, DropView, Explain, ExprSchemable, Filter,
-    LogicalPlan, LogicalPlanBuilder, PlanType, SetVariable, ToStringifiedPlan, WriteOp,
+    DescribeTable, DmlStatement, DropTable, DropView, EmptyRelation, Explain,
+    ExprSchemable, Filter, LogicalPlan, LogicalPlanBuilder, PlanType, SetVariable,
+    ToStringifiedPlan, WriteOp,
 };
 use sqlparser::ast;
 use sqlparser::ast::{
-    Assignment, Expr as SQLExpr, Expr, Ident, ObjectName, ObjectType, Query, SchemaName,
-    SetExpr, ShowCreateObject, ShowStatementFilter, Statement, TableFactor,
+    Assignment, Expr as SQLExpr, Expr, Ident, ObjectName, ObjectType, OrderByExpr, Query,
+    SchemaName, SetExpr, ShowCreateObject, ShowStatementFilter, Statement, TableFactor,
     TableWithJoins, UnaryOperator, Value,
 };
+
 use sqlparser::parser::ParserError::ParserError;
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::sync::Arc;
@@ -114,7 +116,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             } => self.set_variable_to_plan(local, hivevar, &variable, value),
 
             Statement::CreateTable {
-                query: Some(query),
+                query,
                 name,
                 columns,
                 constraints,
@@ -127,42 +129,66 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 && table_properties.is_empty()
                 && with_options.is_empty() =>
             {
-                let plan = self.query_to_plan(*query, planner_context)?;
-                let input_schema = plan.schema();
+                match query {
+                    Some(query) => {
+                        let plan = self.query_to_plan(*query, planner_context)?;
+                        let input_schema = plan.schema();
 
-                let plan = if !columns.is_empty() {
-                    let schema = self.build_schema(columns)?.to_dfschema_ref()?;
-                    if schema.fields().len() != input_schema.fields().len() {
-                        return Err(DataFusionError::Plan(format!(
+                        let plan = if !columns.is_empty() {
+                            let schema = self.build_schema(columns)?.to_dfschema_ref()?;
+                            if schema.fields().len() != input_schema.fields().len() {
+                                return Err(DataFusionError::Plan(format!(
                             "Mismatch: {} columns specified, but result has {} columns",
                             schema.fields().len(),
                             input_schema.fields().len()
                         )));
-                    }
-                    let input_fields = input_schema.fields();
-                    let project_exprs = schema
-                        .fields()
-                        .iter()
-                        .zip(input_fields)
-                        .map(|(field, input_field)| {
-                            cast(col(input_field.name()), field.data_type().clone())
-                                .alias(field.name())
-                        })
-                        .collect::<Vec<_>>();
-                    LogicalPlanBuilder::from(plan.clone())
-                        .project(project_exprs)?
-                        .build()?
-                } else {
-                    plan
-                };
+                            }
+                            let input_fields = input_schema.fields();
+                            let project_exprs = schema
+                                .fields()
+                                .iter()
+                                .zip(input_fields)
+                                .map(|(field, input_field)| {
+                                    cast(
+                                        col(input_field.name()),
+                                        field.data_type().clone(),
+                                    )
+                                    .alias(field.name())
+                                })
+                                .collect::<Vec<_>>();
+                            LogicalPlanBuilder::from(plan.clone())
+                                .project(project_exprs)?
+                                .build()?
+                        } else {
+                            plan
+                        };
 
-                Ok(LogicalPlan::CreateMemoryTable(CreateMemoryTable {
-                    name: self.object_name_to_table_reference(name)?,
-                    input: Arc::new(plan),
-                    if_not_exists,
-                    or_replace,
-                }))
+                        Ok(LogicalPlan::CreateMemoryTable(CreateMemoryTable {
+                            name: self.object_name_to_table_reference(name)?,
+                            input: Arc::new(plan),
+                            if_not_exists,
+                            or_replace,
+                        }))
+                    }
+
+                    None => {
+                        let schema = self.build_schema(columns)?.to_dfschema_ref()?;
+                        let plan = EmptyRelation {
+                            produce_one_row: false,
+                            schema,
+                        };
+                        let plan = LogicalPlan::EmptyRelation(plan);
+
+                        Ok(LogicalPlan::CreateMemoryTable(CreateMemoryTable {
+                            name: self.object_name_to_table_reference(name)?,
+                            input: Arc::new(plan),
+                            if_not_exists,
+                            or_replace,
+                        }))
+                    }
+                }
             }
+
             Statement::CreateView {
                 or_replace,
                 name,
@@ -181,10 +207,6 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                     definition: sql,
                 }))
             }
-            Statement::CreateTable { .. } => Err(DataFusionError::NotImplemented(
-                "Only `CREATE TABLE table_name AS SELECT ...` statement is supported"
-                    .to_string(),
-            )),
             Statement::ShowCreate { obj_type, obj_name } => match obj_type {
                 ShowCreateObject::Table => self.show_create_table_to_plan(obj_name),
                 _ => Err(DataFusionError::NotImplemented(
@@ -413,9 +435,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         let DescribeTableStmt { table_name } = statement;
         let table_ref = self.object_name_to_table_reference(table_name)?;
 
-        let table_source = self
-            .schema_provider
-            .get_table_provider((&table_ref).into())?;
+        let table_source = self.schema_provider.get_table_provider(table_ref)?;
 
         let schema = table_source.schema();
 
@@ -425,6 +445,40 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }))
     }
 
+    fn build_order_by(
+        &self,
+        order_exprs: Vec<OrderByExpr>,
+        schema: &DFSchemaRef,
+        planner_context: &mut PlannerContext,
+    ) -> Result<Vec<datafusion_expr::Expr>> {
+        // Ask user to provide a schema if schema is empty.
+        if !order_exprs.is_empty() && schema.fields().is_empty() {
+            return Err(DataFusionError::Plan(
+                "Provide a schema before specifying the order while creating a table."
+                    .to_owned(),
+            ));
+        }
+        // Convert each OrderByExpr to a SortExpr:
+        let result = order_exprs
+            .into_iter()
+            .map(|e| self.order_by_to_sort_expr(e, schema, planner_context))
+            .collect::<Result<Vec<_>>>()?;
+        // Verify that columns of all SortExprs exist in the schema:
+        for expr in result.iter() {
+            for column in expr.to_columns()?.iter() {
+                if !schema.has_column(column) {
+                    // Return an error if any column is not in the schema:
+                    return Err(DataFusionError::Plan(format!(
+                        "Column {column} is not in schema"
+                    )));
+                }
+            }
+        }
+
+        // If all SortExprs are valid, return them as an expression vector
+        Ok(result)
+    }
+
     /// Generate a logical plan from a CREATE EXTERNAL TABLE statement
     fn external_table_to_plan(
         &self,
@@ -441,6 +495,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             table_partition_cols,
             if_not_exists,
             file_compression_type,
+            order_exprs,
             options,
         } = statement;
 
@@ -461,12 +516,16 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
 
         let schema = self.build_schema(columns)?;
+        let df_schema = schema.to_dfschema_ref()?;
+
+        let ordered_exprs =
+            self.build_order_by(order_exprs, &df_schema, &mut PlannerContext::new())?;
 
         // External tables do not support schemas at the moment, so the name is just a table name
-        let name = OwnedTableReference::Bare { table: name };
+        let name = OwnedTableReference::bare(name);
 
         Ok(LogicalPlan::CreateExternalTable(PlanCreateExternalTable {
-            schema: schema.to_dfschema_ref()?,
+            schema: df_schema,
             name,
             location,
             file_type,
@@ -476,6 +535,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             if_not_exists,
             definition,
             file_compression_type,
+            order_exprs: ordered_exprs,
             options,
         }))
     }
@@ -634,9 +694,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
         // Do a table lookup to verify the table exists
         let table_ref = self.object_name_to_table_reference(table_name.clone())?;
-        let provider = self
-            .schema_provider
-            .get_table_provider((&table_ref).into())?;
+        let provider = self.schema_provider.get_table_provider(table_ref.clone())?;
         let schema = (*provider.schema()).clone();
         let schema = DFSchema::try_from(schema)?;
         let scan =
@@ -652,9 +710,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 let schema = Arc::new(schema.clone());
                 let mut using_columns = HashSet::new();
                 expr_to_columns(&filter_expr, &mut using_columns)?;
-                let filter_expr = normalize_col_with_schemas(
+                let filter_expr = normalize_col_with_schemas_and_ambiguity_check(
                     filter_expr,
-                    &[&schema],
+                    &[&[&schema]],
                     &[using_columns],
                 )?;
                 LogicalPlan::Filter(Filter::try_new(filter_expr, Arc::new(scan))?)
@@ -688,7 +746,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         let table_name = self.object_name_to_table_reference(table_name)?;
         let provider = self
             .schema_provider
-            .get_table_provider((&table_name).into())?;
+            .get_table_provider(table_name.clone())?;
         let arrow_schema = (*provider.schema()).clone();
         let table_schema = Arc::new(DFSchema::try_from(arrow_schema)?);
         let values = table_schema.fields().iter().map(|f| {
@@ -737,9 +795,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 )?;
                 let mut using_columns = HashSet::new();
                 expr_to_columns(&filter_expr, &mut using_columns)?;
-                let filter_expr = normalize_col_with_schemas(
+                let filter_expr = normalize_col_with_schemas_and_ambiguity_check(
                     filter_expr,
-                    &[&table_schema],
+                    &[&[&table_schema]],
                     &[using_columns],
                 )?;
                 LogicalPlan::Filter(Filter::try_new(filter_expr, Arc::new(scan))?)
@@ -790,7 +848,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         let table_name = self.object_name_to_table_reference(table_name)?;
         let provider = self
             .schema_provider
-            .get_table_provider((&table_name).into())?;
+            .get_table_provider(table_name.clone())?;
         let arrow_schema = (*provider.schema()).clone();
         let table_schema = DFSchema::try_from(arrow_schema)?;
 
@@ -896,9 +954,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
         // Do a table lookup to verify the table exists
         let table_ref = self.object_name_to_table_reference(sql_table_name)?;
-        let _ = self
-            .schema_provider
-            .get_table_provider((&table_ref).into())?;
+        let _ = self.schema_provider.get_table_provider(table_ref)?;
 
         // treat both FULL and EXTENDED as the same
         let select_list = if full || extended {
@@ -934,9 +990,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
         // Do a table lookup to verify the table exists
         let table_ref = self.object_name_to_table_reference(sql_table_name)?;
-        let _ = self
-            .schema_provider
-            .get_table_provider((&table_ref).into())?;
+        let _ = self.schema_provider.get_table_provider(table_ref)?;
 
         let query = format!(
             "SELECT table_catalog, table_schema, table_name, definition FROM information_schema.views WHERE {where_clause}"
